"""Package name and version."""

from importlib.metadata import PackageNotFoundError, version

PROGRAM_NAME = PACKAGE_NAME = "nudebomb"


def get_version():
    """Get the current installed comicbox version."""
    try:
        v = version(PACKAGE_NAME)
    except PackageNotFoundError:
        v = "dev"
    return v


VERSION = get_version()



"""Test Track class."""

import json

from nudebomb.track import Track

from .util import read

__all__ = ()


class TestTrack:
    """Test Track."""

    def test_video_track(self):
        """Test video track."""
        data = json.loads(read("clean-tracks.json"))["tracks"][0]
        track = Track(data)
        assert str(track) == "Track #0: und - MPEG-4p10/AVC/h.264"

    def test_audio_track(self):
        """Test audio track."""
        data = json.loads(read("clean-tracks.json"))["tracks"][1]
        track = Track(data)
        assert str(track) == "Track #2: eng - AC-3"

    def test_subtitle_track(self):
        """Test subtitile track."""
        data = json.loads(read("clean-tracks.json"))["tracks"][2]
        track = Track(data)
        assert str(track) == "Track #5: eng - SubRip/SRT"



"""Command line interface for nudebomb."""

from argparse import Action, ArgumentParser, Namespace, RawDescriptionHelpFormatter

from termcolor import colored

from nudebomb.config import get_config
from nudebomb.version import VERSION
from nudebomb.walk import Walk


class CommaListAction(Action):
    """Split arguments by commas into a list."""

    DELINEATOR = ","

    def __call__(self, _, namespace, values, _option_string=None):
        """Split by delineator and assign to dest variable."""
        items = values.strip().split(self.DELINEATOR)
        setattr(namespace, self.dest, items)


def get_arguments(params=None):
    """Command line interface."""
    description = "Strips unnecessary tracks from MKV files."
    epilog = (
        "Dot color key:\n"
        + colored("\tMKV ignored", "white", attrs=["dark"])
        + "\n"
        + colored("\tMKV stripped tracks")
        + "\n"
        + colored("\tMKV already stripped or timestamp unchanged", "green")
        + "\n"
        + colored("\tMKV not remuxed on dry run", "black", attrs=["bold"])
        + "\n"
        + colored("\tWarning", "yellow")
        + "\n"
        + colored("\tError", "red")
        + "\n"
    )
    parser = ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "-d",
        "--dry-run",
        action="store_true",
        help="Enable mkvmerge dry run for testing.",
    )
    parser.add_argument(
        "-b",
        "--mkvmerge-bin",
        action="store",
        help="The path to the MKVMerge executable.",
    )
    parser.add_argument(
        "-l",
        "--languages",
        action=CommaListAction,
        help="Comma-separated list of audio and subtitle languages to retain. "
        "e.g. eng,fre.",
    )
    parser.add_argument(
        "-U",
        "--strip-und-language",
        action="store_true",
        help="Strip the 'und' undetermined or untagged language tracks. "
        "By default nudebomb does not strip these tracks.",
    )
    parser.add_argument(
        "-s",
        "--sub-languages",
        action=CommaListAction,
        required=False,
        help="Comma-separated list of subtitle specific languages to retain. "
        "Supersedes --languages.",
    )
    parser.add_argument(
        "-S",
        "--no-subtitles",
        action="store_false",
        dest="subtitles",
        help="If no subtitles match the languages to retain, strip all subtitles. "
        "By default nudebomb keeps all subtitles if no subtitles match specified "
        "languages.",
    )
    parser.add_argument(
        "-i",
        "--ignore",
        action=CommaListAction,
        dest="ignore",
        help="List of globs to ignore.",
    )
    parser.add_argument(
        "-L",
        "--no-symlinks",
        action="store_false",
        dest="symlinks",
        help="Do not follow symlinks for files and directories",
    )
    parser.add_argument(
        "-T",
        "--no-title",
        action="store_false",
        dest="title",
        help="Do not rewrite the metadata title with the filename stem when remuxing.",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="count",
        help="Verbose output. Can be used multiple times for noisier output.",
    )
    parser.add_argument(
        "-q",
        "--quiet",
        action="store_const",
        dest="verbose",
        const=0,
        help="Display little to no output.",
    )
    parser.add_argument(
        "-r",
        "--recurse",
        action="store_true",
        help="Recurse through all paths on the command line.",
    )
    parser.add_argument(
        "-t",
        "--timestamps",
        action="store_true",
        help="Read and write timestamps to strip only files that have been "
        "modified since the last run.",
    )
    parser.add_argument(
        "-C",
        "--timestamps-no-check-config",
        dest="timestamps_check_config",
        action="store_false",
        default=True,
        help="Do not compare program config options with loaded timestamps.",
    )
    parser.add_argument(
        "-c", "--config", action="store", help="Alternate config file path"
    )
    parser.add_argument(
        "-A",
        "--after",
        action="store",
        dest="after",
        help="Only strip mkvs after the specified timestamp. "
        "Supersedes recorded timestamp files. Can be an epoch number or "
        "datetime string.",
    )
    parser.add_argument(
        "-V", "--version", action="version", version=f"%(prog)s {VERSION}"
    )
    parser.add_argument(
        "paths",
        metavar="path",
        type=str,
        nargs="+",
        help="Where your MKV files are stored. Can be a directories or files.",
    )

    # Parse the list of given arguments
    if params is not None:
        params = params[1:]
    nns = parser.parse_args(params)
    return Namespace(nudebomb=nns)


def main(args: tuple[str, ...] | None = None):
    """Process command line arguments, config and walk inputs."""
    arguments = get_arguments(args)
    config = get_config(arguments)
    # Iterate over all found mkv files
    walker = Walk(config)
    walker.run()


if __name__ == "__main__":
    main()



"""Confuse config for comicbox."""

import os
import sys
from argparse import Namespace
from pathlib import Path
from platform import system
from time import mktime

from confuse import Configuration
from confuse.templates import AttrDict, Integer, MappingTemplate, Optional, Sequence
from dateutil.parser import parse
from termcolor import cprint

from nudebomb.version import PROGRAM_NAME

TEMPLATE = MappingTemplate(
    {
        PROGRAM_NAME: MappingTemplate(
            {
                "after": Optional(str),
                "dry_run": bool,
                "ignore": Sequence(str),
                "languages": Sequence(str),
                "mkvmerge_bin": Optional(str),
                "paths": Sequence(str),
                "recurse": bool,
                "strip_und_language": bool,
                "sub_languages": Optional(Sequence(str)),
                "subtitles": bool,
                "symlinks": bool,
                "timestamps": bool,
                "timestamps_check_config": bool,
                "title": bool,
                "verbose": Integer(),
            }
        )
    }
)
TIMESTAMPS_CONFIG_KEYS = {
    "languages",
    "mkvmerge_bin",
    "recurse",
    "strip_und_language",
    "sub_languages",
    "subtitles",
    "symlinks",
    "title",
}

if system() == "Windows":
    os.system("color")  # noqa: S605, S607


def _set_after(config) -> None:
    after = config[PROGRAM_NAME]["after"].get()
    if after is None:
        return

    try:
        timestamp = float(after)
    except ValueError:
        after_dt = parse(after)
        timestamp = mktime(after_dt.timetuple())

    config[PROGRAM_NAME]["after"].set(timestamp)


def _set_default_mkvmerge_bin(config):
    if config[PROGRAM_NAME]["mkvmerge_bin"].get():
        return

    if system() == "Windows":
        config[PROGRAM_NAME]["mkvmerge_bin"].set(
            "C:\\\\Program Files\\MKVToolNix\\mkvmerge.exe"
        )
    else:
        config[PROGRAM_NAME]["mkvmerge_bin"].set("mkvmerge")


def _set_unique_lang_list(config, key):
    if config[PROGRAM_NAME][key].get() is not None:
        items = set(config[PROGRAM_NAME][key].get())
        if not config[PROGRAM_NAME]["strip_und_language"].get():
            items.add("und")
        config[PROGRAM_NAME][key].set(sorted(frozenset(items)))


def _set_languages(config):
    _set_unique_lang_list(config, "languages")
    if not config[PROGRAM_NAME]["languages"].get():
        cprint(
            "Nudebomb will not run unless you set languages to keep on the "
            "command line, environment variables or config files.",
            "red",
        )
        sys.exit(1)


def _set_ignore(config) -> None:
    """Remove duplicates from the ignore list."""
    ignore: list[str] = config[PROGRAM_NAME]["ignore"].get(list)
    config[PROGRAM_NAME]["ignore"].set(tuple(sorted(set(ignore))))


def _set_timestamps(config) -> None:
    """Set the timestamp attribute."""
    timestamps = config[PROGRAM_NAME]["timestamps"].get(bool) and not config[
        PROGRAM_NAME
    ]["dry_run"].get(bool)
    config[PROGRAM_NAME]["timestamps"].set(timestamps)


def get_config(args: Namespace | None = None, modname=PROGRAM_NAME) -> AttrDict:
    """Get the config dict, layering env and args over defaults."""
    config = Configuration(PROGRAM_NAME, modname=modname, read=False)
    try:
        config.read()
    except Exception as exc:
        cprint(f"WARNING: {exc}")
    if args and args.nudebomb and args.nudebomb.config:
        config.set_file(args.nudebomb.config)
    config.set_env()
    if args:
        config.set_args(args)
    _set_languages(config)
    _set_after(config)
    _set_default_mkvmerge_bin(config)
    _set_unique_lang_list(config, "sub_languages")
    _set_ignore(config)
    _set_timestamps(config)
    ad = config.get(TEMPLATE)
    if not isinstance(ad, AttrDict):
        raise TypeError
    ad.paths = sorted(
        frozenset(str(Path(path).resolve()) for path in ad.nudebomb.paths)
    )
    return ad.nudebomb



"""Walk directory trees and strip mkvs."""

from copy import deepcopy
from pathlib import Path

from termcolor import cprint
from treestamps import Grovestamps, GrovestampsConfig
from treestamps.tree import Treestamps

from nudebomb.config import TIMESTAMPS_CONFIG_KEYS
from nudebomb.langfiles import LangFiles
from nudebomb.mkv import MKVFile
from nudebomb.version import PROGRAM_NAME


class Walk:
    """Directory traversal class."""

    def __init__(self, config):
        """Initialize."""
        self._config = config
        self._langfiles = LangFiles(config)

    def _is_path_suffix_not_mkv(self, path: Path) -> bool:
        """Return if the suffix should skipped."""
        if path.suffix == ".mkv":
            return False
        if self._config.verbose > 2:  # noqa: PLR2004
            cprint(f"Skip suffix is not 'mkv' f{path}")
        return True

    def _is_path_ignored(self, path: Path) -> bool:
        """Return if path should be ignored."""
        if any(path.match(ignore_glob) for ignore_glob in self._config.ignore):
            if self._config.verbose > 1:
                cprint(f"Skip ignored {path}", "white", attrs=["dark"])
            elif self._config.verbose:
                cprint(".", "white", attrs=["dark"], end="")
            return True
        return False

    def _is_path_before_timestamp(self, top_path: Path, path: Path) -> bool:
        """Return if the file was last updated before the timestamp."""
        if self._config.after:
            mtime = self._config.after
        elif self._config.timestamps:
            mtime = self._timestamps.get(top_path, {}).get(path)
        else:
            mtime = None

        if mtime is not None and mtime > path.stat().st_mtime:
            color = "green"
            if self._config.verbose > 1:
                cprint(f"Skip by timestamp {path}", color, attrs=["dark"])
            elif self._config.verbose:
                cprint(".", color, end="")
            return True
        return False

    def _is_path_skippable_symlink(self, path: Path):
        if not self._config.symlinks and path.is_symlink():
            color = "white"
            attrs = ["dark"]
            if self._config.verbose > 1:
                cprint(f"Skip symlink {path}", color, attrs=attrs)
            elif self._config.verbose:
                cprint(".", color, attrs=attrs, end="")
            return True
        return False

    def strip_path(self, top_path, path):
        """Strip a single mkv file."""
        dir_path = Treestamps.get_dir(path)
        config = deepcopy(self._config)
        config.languages = self._langfiles.get_langs(top_path, dir_path)
        mkv_obj = MKVFile(config, path)
        mkv_obj.remove_tracks()
        if self._config.timestamps:
            self._timestamps[top_path].set(path)

    def walk_dir(self, top_path, dir_path):
        """Walk a directory."""
        if not self._config.recurse:
            return

        filenames = []

        for filename in sorted(dir_path.iterdir()):
            if filename.is_dir():
                self.walk_file(top_path, filename)
            else:
                filenames.append(filename)

        for path in filenames:
            self.walk_file(top_path, path)

        if self._config.timestamps:
            timestamps = self._timestamps[top_path]
            timestamps.set(dir_path, compact=True)

    def walk_file(self, top_path, path):
        """Walk a file."""
        if self._is_path_ignored(path):
            return
        if self._is_path_skippable_symlink(path):
            return
        if path.is_dir():
            self.walk_dir(top_path, path)
        else:
            if self._is_path_suffix_not_mkv(path):
                return
            if self._is_path_before_timestamp(top_path, path):
                return
            self.strip_path(top_path, path)

    def print_info(self):
        """Print intentions before we begin."""
        langs = ", ".join(sorted(self._config.languages))
        audio = "audio " if self._config.sub_languages else ""
        cprint(f"Stripping {audio}languages except {langs}.")
        if self._config.sub_languages:
            sub_langs = ", ".join(sorted(self._config.sub_languages))
            cprint(f"Stripping subtitle languages except {sub_langs}.")

        cprint("Searching for MKV files to process", end="")
        if self._config.verbose > 1:
            cprint(":")

    def run(self):
        """Run the stripper against all configured paths."""
        self.print_info()

        if self._config.timestamps:
            copse_config = GrovestampsConfig(
                PROGRAM_NAME,
                paths=self._config.paths,
                verbose=self._config.verbose,
                symlinks=self._config.symlinks,
                ignore=self._config.ignore,
                check_config=self._config.timestamps_check_config,
                program_config=self._config,
                program_config_keys=TIMESTAMPS_CONFIG_KEYS,
            )
            self._timestamps = Grovestamps(copse_config)

        for path_str in self._config.paths:
            path = Path(path_str)
            top_path = Treestamps.get_dir(path)
            self.walk_file(top_path, path)
        if not self._config.verbose > 1:
            cprint("done.")

        if self._config.timestamps:
            self._timestamps.dump()



"""Integration tests."""

import shutil
from pathlib import Path

from nudebomb.cli import main
from nudebomb.mkv import MKVFile

from .test_mkv import assert_eng_und_only
from .util import SRC_DIR, TEST_FN, mkv_tracks

TEST_DIR = Path("/tmp/nudebomb.test.integration")  # noqa:S108
TEST_MKV = TEST_DIR / TEST_FN

__all__ = ()


class TestIntegrated:
    """Integrated tests."""

    def setup_method(self):
        """Set up tests."""
        shutil.rmtree(TEST_DIR, ignore_errors=True)
        TEST_DIR.mkdir()
        src_path = SRC_DIR / TEST_FN
        self.dest_path = TEST_DIR / TEST_FN
        shutil.copy(src_path, self.dest_path)
        self.src_tracks = mkv_tracks(self.dest_path)

    def teardown_method(self):
        """Tear down tests."""
        if TEST_DIR.exists():
            shutil.rmtree(TEST_DIR, ignore_errors=True)

    def test_dry_run(self):
        """Test dry run."""
        main(("nudebomb", "-l", "eng,und", "-d", str(self.dest_path)))
        out_tracks = mkv_tracks(self.dest_path)
        assert out_tracks == self.src_tracks

    def test_run(self):
        """Test run."""
        main(("nudebomb", "-l", "eng,und", "-r", str(TEST_DIR)))
        out_tracks = mkv_tracks(self.dest_path)
        assert_eng_und_only(out_tracks)

    def test_fail(self):
        """Test fail."""
        main(("nudebomb", "-l", "eng", str(TEST_DIR)))
        out_tracks = mkv_tracks(self.dest_path)
        assert out_tracks == self.src_tracks

    def test_strip_all_subs(self):
        """Test strib all subs."""
        main(("nudebomb", "-l", "eng,und", "-s", "''", "-S", "-U", "-r", str(TEST_DIR)))
        out_tracks = mkv_tracks(self.dest_path)
        for track in out_tracks:
            track_type = track.get("type")
            if track_type == MKVFile.SUBTITLE_TRACK_NAME:
                msg = f"subtitle track should not exist: {track}"
                raise AssertionError(msg)



"""Test MKVFile object."""

import os
import shutil
from pathlib import Path

from nudebomb.config import get_config
from nudebomb.mkv import MKVFile

from .util import SRC_PATH, TEST_FN, mkv_tracks

__all__ = ()

TEST_DIR = Path("/tmp/nudebomb.test_remux")  # noqa: S108
TEST_MKV = TEST_DIR / TEST_FN


def assert_eng_und_only(out_tracks):
    """Asset english and undefined only tracks."""
    audio_count = 0
    subs_count = 0
    for track in out_tracks:
        track_type = track.get("type")
        if track_type not in MKVFile.REMOVABLE_TRACK_NAMES:
            continue
        lang = track["properties"]["language"]
        print(track_type, lang)  # noqa: T201
        assert lang in ["und", "eng"]
        if track_type == MKVFile.SUBTITLE_TRACK_NAME:
            subs_count += 1
        elif track_type == MKVFile.AUDIO_TRACK_NAME:
            audio_count += 1
        else:
            msg = f"Bad track type: {track_type}"
            raise AssertionError(msg)
    assert audio_count == 2  # noqa: PLR2004
    assert subs_count == 2  # noqa: PLR2004


class TestMkv:
    """Test MKV."""

    def setup_method(self):
        """Set up method."""
        shutil.rmtree(TEST_DIR, ignore_errors=True)
        TEST_DIR.mkdir()
        shutil.copy(SRC_PATH, TEST_MKV)
        self.src_tracks = mkv_tracks(TEST_MKV)
        os.environ["NUDEBOMB_NUDEBOMB__LANGUAGES__0"] = "und"
        os.environ["NUDEBOMB_NUDEBOMB__LANGUAGES__1"] = "eng"

    def teardown_method(self):
        """Tear down method."""
        shutil.rmtree(TEST_DIR)

    def test_dry_run(self):
        """Test dry run."""
        config = get_config()
        config.dry_run = True
        mkvfile = MKVFile(config, TEST_MKV)
        mkvfile.remove_tracks()
        out_tracks = mkv_tracks(TEST_MKV)

        assert out_tracks == self.src_tracks

    def test_run(self):
        """Test run."""
        config = get_config()
        mkvfile = MKVFile(config, TEST_MKV)
        mkvfile.remove_tracks()
        out_tracks = mkv_tracks(TEST_MKV)
        assert_eng_und_only(out_tracks)

    def test_fail(self):
        """Test fail."""
        config = get_config()
        config.languages = ["xxx"]
        mkvfile = MKVFile(config, TEST_MKV)
        mkvfile.remove_tracks()
        out_tracks = mkv_tracks(TEST_MKV)

        assert out_tracks == self.src_tracks



"""MKV Track metadata."""


class Track:
    """MKV track metadata."""

    def __init__(self, track_data):
        """Initialize."""
        self.type = track_data["type"]
        self.id = track_data["id"]
        self.lang = track_data["properties"].get("language", "und")
        self.codec = track_data["codec"]

    def __str__(self):
        """Represetnd as a string."""
        return f"Track #{self.id}: {self.lang} - {self.codec}"



"""MKV file operations."""

import json
import subprocess
import sys
from pathlib import Path

from termcolor import cprint

from nudebomb.langfiles import lang_to_alpha3
from nudebomb.track import Track


class MKVFile:
    """Strips matroska files of unwanted audio and subtitles."""

    VIDEO_TRACK_NAME = "video"
    AUDIO_TRACK_NAME = "audio"
    SUBTITLE_TRACK_NAME = "subtitles"
    REMOVABLE_TRACK_NAMES = (AUDIO_TRACK_NAME, SUBTITLE_TRACK_NAME)

    def __init__(self, config, path):
        """Initialize."""
        self._config = config
        self.path = Path(path)
        self._init_track_map()

    def _init_track_map(self):
        self._track_map = {}

        # Ask mkvmerge for the json info
        command = (self._config.mkvmerge_bin, "-J", str(self.path))
        proc = subprocess.run(  # noqa: S603
            command,
            capture_output=True,
            check=True,
            text=True,
        )

        # Process the json response
        json_data = json.loads(proc.stdout)
        if errors := json_data.get("errors"):
            for error in errors:
                cprint(f"\nERROR: {error}", "red")
        if warnings := json_data.get("warnings"):
            for warning in warnings:
                cprint(f"\nWARNING: {warning}", "yellow")
        tracks = json_data.get("tracks")
        if not tracks:
            cprint(
                "\nWARNING: No tracks. Might not be a valid matroshka "
                f"video file: {self.path}",
                "yellow",
            )
            return

        # load into our map.
        for track_data in tracks:
            track_obj = Track(track_data)
            if track_obj.type not in self._track_map:
                self._track_map[track_obj.type] = []
            self._track_map[track_obj.type].append(track_obj)

    def _filtered_tracks(self, track_type):
        """Return a tuple consisting of tracks to keep and tracks to remove."""
        if track_type == self.SUBTITLE_TRACK_NAME and self._config.sub_languages:
            languages_to_keep = self._config.sub_languages
        else:
            languages_to_keep = self._config.languages

        # Lists of track to keep & remove
        remove = []
        keep = []
        # Iterate through all tracks to find which track to keep or remove
        tracks = self._track_map.get(track_type, [])
        for track in tracks:
            if self._config.verbose > 1:
                cprint(
                    f"\t{track_type}: {track.id} {track.lang}", "white", attrs=["dark"]
                )
            track_lang = lang_to_alpha3(track.lang)
            if track_lang in languages_to_keep:
                # Tracks we want to keep
                keep.append(track)
            else:
                # Tracks we want to remove
                remove.append(track)

        if not keep and (track_type == self.AUDIO_TRACK_NAME or self._config.subtitles):
            # Never remove all audio
            # Do not remove all subtitles without option set.
            keep = remove
            remove = []

        return keep, remove

    def _extend_track_command(self, track_type, output, command, num_remove_ids):
        keep, remove = self._filtered_tracks(track_type)

        # Build the keep tracks options
        keep_ids = set()

        output += f"Retaining {track_type} track(s):\n"
        for count, track in enumerate(keep):
            keep_ids.add(str(track.id))
            output += f"   {track}\n"

            # Set the first track as default
            command += [
                "--default-track",
                ":".join((str(track.id), "0" if count else "1")),
            ]

        # Set which tracks are to be kept
        if keep_ids:
            prefix = track_type
            if track_type == self.SUBTITLE_TRACK_NAME:
                prefix = prefix[:-1]
            command += [f"--{prefix}-tracks", ",".join(sorted(keep_ids))]
        elif track_type == self.SUBTITLE_TRACK_NAME:
            command += ["--no-subtitles"]
        else:
            cprint(f"WARNING: No tracks to remove from {self.path}", "yellow")
            return output, command, num_remove_ids

        # Report what tracks will be removed
        output += f"Removing {track_type} track(s):\n"
        for track in remove:
            output += f"   {track}\n"

        output += "----------------------------\n"

        num_remove_ids += len(remove)

        return output, command, num_remove_ids

    @staticmethod
    def _remux_file(command):
        """Remux a mkv file with the given parameters."""
        sys.stdout.write("Progress 0%")
        sys.stdout.flush()

        # Call command to remux file
        with subprocess.Popen(  # noqa: S603
            command,
            stdout=subprocess.PIPE,
            bufsize=1,
            text=True,
        ) as process:
            if process.stdout:
                for line in iter(process.stdout.readline, ""):
                    if "progress" in line.lower():
                        outline = f"\r{line.strip()}"
                        sys.stdout.write(outline)
                        sys.stdout.flush()
            print(flush=True)  # noqa: T201

            # Check if return code indicates an error
            if retcode := process.poll():
                kwargs = {}
                if process.stdout is not None:
                    kwargs["output"] = process.stdout
                raise subprocess.CalledProcessError(retcode, command, **kwargs)

    def remove_tracks(self):
        """Remove the unwanted tracks."""
        if not self._track_map:
            cprint(
                f"ERROR: not removing tracks from mkv with no tracks: {self.path}",
                "red",
            )
            return
        if self._config.verbose > 1:
            cprint(f"Checking {self.path}:", "white", attrs=["dark"])
        # The command line args required to remux the mkv file
        output = f"\nRemuxing: {self.path}\n"
        output += "============================\n"

        # Output the remuxed file to a temp tile, This will protect
        # the original file from been corrupted if anything goes wrong
        tmp_path = self.path.with_suffix(self.path.suffix + ".tmp")
        command = [
            self._config.mkvmerge_bin,
            "--output",
            str(tmp_path),
        ]
        if self._config.title:
            command += [
                "--title",
                self.path.stem,
            ]

        # Iterate all tracks and mark which tracks are to be kept
        num_remove_ids = 0
        for track_type in self.REMOVABLE_TRACK_NAMES:
            output, command, num_remove_ids = self._extend_track_command(
                track_type, output, command, num_remove_ids
            )
        command += [(str(self.path))]

        if not num_remove_ids:
            if self._config.verbose > 1:
                cprint(f"\tAlready stripped {self.path}", "green", attrs=["dark"])
            elif self._config.verbose:
                cprint(".", "green", attrs=["bold"], end="")
            return

        try:
            cprint(output, flush=True)
            if self._config.dry_run:
                cprint("\tNot remuxing on dry run {self.path}", "black", attrs=["bold"])
            else:
                self._remux_file(command)
                tmp_path.replace(self.path)
        except Exception as exc:
            cprint(f"ERROR: {exc}", "red")
            tmp_path.unlink(missing_ok=True)



"""Common test utilities."""

import json
import subprocess
from pathlib import Path

TEST_FN = "test5.mkv"
SRC_DIR = Path("tests/test_files")
SRC_PATH = SRC_DIR / TEST_FN

__all__ = ()


def mkv_tracks(path):
    """Get tracks from mkv."""
    cmd = ("mkvmerge", "-J", str(path))
    proc = subprocess.run(cmd, check=True, capture_output=True, text=True)  # noqa: S603
    data = json.loads(proc.stdout)
    return data.get("tracks")


def read(filename):
    """Open data file and return contents."""
    path = Path(__file__).parent / "mockdata" / filename
    with path.open("r") as stream:
        return stream.read()



"""Module for reading lang files."""

from contextlib import suppress

import pycountry
from termcolor import cprint

LANGS_FNS = ("lang", "langs", ".lang", ".langs")


def lang_to_alpha3(lang):
    """Convert languages to ISO-639-1 (alpha2) format."""
    if not lang:
        lang = "und"
    elif len(lang) == 3:  # noqa: PLR2004
        pass
    elif len(lang) == 2:  # noqa: PLR2004
        with suppress(Exception):
            if lo := pycountry.languages.get(alpha_2=lang):
                lang = lo.alpha_3
    else:
        cprint(
            f"WARNING: Languages should be in two or three letter format: {lang}",
            "yellow",
        )

    return lang


class LangFiles:
    """Process nudebomb langfiles."""

    def __init__(self, config):
        """Initialize."""
        self._config = config
        self._lang_roots = {}
        langs = set()
        for lang in self._config.languages:
            langs.add(lang_to_alpha3(lang))
        self._languages = frozenset(langs)

    def read_lang_files(self, path):
        """
        Read the lang files and parse languages.

        lang_roots is a dictionary to cache paths and languages to avoid
        reparsing the same language files.
        """
        if path not in self._lang_roots:
            self._lang_roots[path] = set()
            for fn in LANGS_FNS:
                langpath = path / fn
                if (
                    not langpath.exists()
                    or not langpath.is_file()
                    or (not self._config.symlinks and langpath.is_symlink())
                ):
                    # ignore is already handled before we get here in walk.py
                    continue
                newlangs = set()
                with langpath.open("r") as langfile:
                    for line in langfile:
                        for lang in line.strip().split(","):
                            newlang = lang_to_alpha3(lang.strip())
                            newlangs.add(newlang)
                if self._config.verbose > 1:
                    newlangs_str = " ,".join(sorted(newlangs))
                    cprint(f"Also keeping {newlangs_str} for {path}", "cyan")
                self._lang_roots[path] |= newlangs

        return self._lang_roots[path]

    def get_langs(self, top_path, path):
        """Get the languages from this dir and parent dirs."""
        langs = self._languages
        while True:
            langs |= self.read_lang_files(path)
            path = path.parent
            if path in (top_path, path.parent):
                break
        return frozenset(langs)
